@@ -1,5 +1,6 @@
 // This file is part of fityk program. Copyright (C) Marcin Wojdyr
-// $Id: NMfit.h,v 1.5 2006/07/10 15:58:50 wojdyr Exp $
+// Licence: GNU General Public License version 2
+// $Id: NMfit.h 264 2007-03-01 10:10:54Z wojdyr $
 
 #ifndef FITYK__NMFIT__H__
 #define FITYK__NMFIT__H__
@@ -33,17 +34,13 @@ private:
     std::vector<Vertex> vertices;
     std::vector<Vertex>::iterator best, s_worst /*second worst*/, worst;
     std::vector<fp> coord_sum;
-    fp min_rel_diff;
-    bool move_all; 
-    char distrib_type;
-    fp move_mult;
     fp volume_factor;
 
     void find_best_worst();
     void change_simplex();
     fp try_new_worst (fp f);
     void compute_coord_sum();
-    bool termination_criteria (int iter);
+    bool termination_criteria (int iter, fp convergence);
     void compute_v (Vertex& v); 
 };
 
