@@ -1,5 +1,6 @@
 // This file is part of fityk program. Copyright (C) 2005 Marcin Wojdyr
-// $Id: ast.cpp,v 1.4 2006/09/12 00:28:45 wojdyr Exp $
+// Licence: GNU General Public License version 2
+// $Id: ast.cpp 264 2007-03-01 10:10:54Z wojdyr $
 
 //  Based on ast_calc example from Boost::Spirit by Daniel Nuffer
 
@@ -23,20 +24,24 @@
 #include <cmath>
 
 #include "common.h"
+#include "ui.h"
 #include "ast.h"
 #include "var.h"
 #include "datatrans.h"
 #include "logic.h"
 #include "numfuncs.h"
+#include "voigt.h"
 
 ////////////////////////////////////////////////////////////////////////////
 using namespace std;
 using namespace boost::spirit;
 
-typedef char const*         iterator_t;
-typedef tree_match<iterator_t> parse_tree_match_t;
-typedef parse_tree_match_t::tree_iterator iter_t;
-typedef parse_tree_match_t::const_tree_iterator const_iter_t;
+//typedef char const*         iterator_t;
+//typedef tree_match<iterator_t> parse_tree_match_t;
+//typedef parse_tree_match_t::tree_iterator iter_t;
+//typedef parse_tree_match_t::const_tree_iterator const_iter_t;
+//typedef tree_match<char const*>::const_tree_iterator const_iter_t;
+//typedef tree_match<char const*>::const_tree_iterator const_tm_iter_t;
 
 ////////////////////////////////////////////////////////////////////////////
 
@@ -69,6 +74,11 @@ string OpTree::str(const vector<string> *vars)
         case OP_LOG10:return "log10("+c1->str(vars) + ")";
         case OP_LN:   return "ln("  + c1->str(vars) + ")";
         case OP_SQRT: return "sqrt("+ c1->str(vars) + ")";
+        case OP_VOIGT: return "voigt("+ c1->str(vars) +","+ c2->str(vars) +")";
+        case OP_DVOIGT_DX: return "dvoigt_dx("+ c1->str(vars) 
+                                                   + "," + c2->str(vars) + ")";
+        case OP_DVOIGT_DY: return "dvoigt_dy("+ c1->str(vars) 
+                                                   + "," + c2->str(vars) + ")";
         case OP_POW:  return c1->str_b(c1->op >= OP_POW, vars) 
                              + "^" + c2->str_b(c2->op >= OP_POW, vars);
         case OP_ADD:  return c1->str(vars) + "+" + c2->str(vars);
@@ -132,9 +142,9 @@ OpTree* OpTree::copy() const
 ////////////////////////////////////////////////////////////////////////////
 
 namespace {
-void do_find_tokens(int tokenID, const_iter_t const &i, vector<string> &vars)
+void do_find_tokens(int tokenID, const_tm_iter_t const &i, vector<string> &vars)
 {
-    for (const_iter_t j = i->children.begin(); j != i->children.end(); ++j) {
+    for (const_tm_iter_t j = i->children.begin(); j != i->children.end(); ++j) {
         if (j->value.id() == tokenID) {
             string v(j->value.begin(), j->value.end());
             if (find(vars.begin(), vars.end(), v) == vars.end())
@@ -149,7 +159,7 @@ void do_find_tokens(int tokenID, const_iter_t const &i, vector<string> &vars)
 vector<string> find_tokens_in_ptree(int tokenID, const tree_parse_info<> &info)
 {
     vector<string> vars;
-    const_iter_t const &root = info.trees.begin();
+    const_tm_iter_t const &root = info.trees.begin();
     if (root->value.id() == tokenID) 
         vars.push_back(string(root->value.begin(), root->value.end()));
     else
@@ -180,7 +190,13 @@ OpTree* do_neg(OpTree *a)
 
 OpTree* do_add(int op, OpTree *a, OpTree *b)
 {
-    if (a->op == 0 && is_eq(a->val, 0.)) { // 0 + t
+    if (a->op == 0 && b->op == 0) { // p + q
+        double val = (op == OP_ADD ? a->val + b->val : a->val - b->val);
+        delete a;
+        delete b;
+        return new OpTree(val);
+    }
+    else if (a->op == 0 && is_eq(a->val, 0.)) { // 0 + t
         delete a;
         if (op == OP_ADD)
             return b;
@@ -191,12 +207,6 @@ OpTree* do_add(int op, OpTree *a, OpTree *b)
         delete b;
         return a;
     }
-    else if (a->op == 0 && b->op == 0) { // p + q
-        double val = (op == OP_ADD ? a->val + b->val : a->val - b->val);
-        delete a;
-        delete b;
-        return new OpTree(val);
-    }
     else if (b->op == OP_NEG) { // t + -u
         OpTree *t = b->remove_c1();
         delete b;
@@ -232,31 +242,42 @@ OpTree* do_sub(OpTree *a, OpTree *b)
 
 OpTree* do_multiply(OpTree *a, OpTree *b)
 {
-    if ((a->op == 0 && b->op == 0)
-        || (a->op == 0 && is_eq(a->val, 0.))
-        || (b->op == 0 && is_eq(b->val, 0.)))
+    if (a->op == 0 && b->op == 0)  // const * const
     {
         double val = a->val * b->val;
         delete a;
         delete b;
         return new OpTree(val);
     }
-    else if (a->op == 0 && is_eq(a->val, 1.)) {
+    else if ((a->op == 0 && is_eq(a->val, 0.))   // 0 * ...
+             || (b->op == 0 && is_eq(b->val, 0.))) // ... * 0
+    {
+        delete a;
+        delete b;
+        return new OpTree(0.);
+    }
+    else if (a->op == 0 && is_eq(a->val, 1.)) {  // 1 * ...
         delete a;
         return b;
     }
-    else if (b->op == 0 && is_eq(b->val, 1.)) {
+    else if (b->op == 0 && is_eq(b->val, 1.)) {  // ... * 1
         delete b;
         return a;
     }
-    else if (a->op == 0 && is_eq(a->val, -1.)) {
+    else if (a->op == 0 && is_eq(a->val, -1.)) { // -1 * ...
         delete a;
         return do_neg(b);
     }
-    else if (b->op == 0 && is_eq(b->val, -1.)) {
+    else if (b->op == 0 && is_eq(b->val, -1.)) { // ... * -1 
         delete b;
         return do_neg(a);
     }
+    // const1 * (const2 / ...)
+    else if (a->op == 0 && b->op == OP_DIV && b->c1->op == 0) { 
+        b->c1->val *= a->val;
+        delete a;
+        return b;
+    }
     else {
         return new OpTree(OP_MUL, a, b);
     }
@@ -264,18 +285,19 @@ OpTree* do_multiply(OpTree *a, OpTree *b)
 
 OpTree* do_divide(OpTree *a, OpTree *b)
 {
-    if (a->op == 0 && is_eq(a->val, 0.)) {
-        delete a;
-        delete b;
-        return new OpTree(0.);
-    }
-    else if ((a->op == 0 && b->op == 0) || (b->op == 0 && is_eq(b->val, 0.)))
+    //no check for division by zero
+    if (a->op == 0 && b->op == 0)
     {
         double val = a->val / b->val;
         delete a;
         delete b;
         return new OpTree(val);
     }
+    else if (a->op == 0 && is_eq(a->val, 0.)) {
+        delete a;
+        delete b;
+        return new OpTree(0.);
+    }
     else if (b->op == 0 && is_eq(b->val, 1.)) {
         delete b;
         return a;
@@ -453,12 +475,18 @@ OpTree* do_digamma(OpTree *a)
 
 OpTree* do_pow(OpTree *a, OpTree *b)
 {
-    if (a->op == 0 && is_eq(a->val, 0.)) {
+    if (a->op == 0 && b->op == 0) {
+        double val = pow(a->val, b->val);
+        delete a;
+        delete b;
+        return new OpTree(val);
+    }
+    else if (a->op == 0 && is_eq(a->val, 0.)) {
         delete a;
         delete b;
         return new OpTree(0.);
     }
-    if ((b->op == 0 && is_eq(b->val, 0.)) 
+    else if ((b->op == 0 && is_eq(b->val, 0.)) 
         || (a->op == 0 && is_eq(a->val, 1.))) {
         delete a;
         delete b;
@@ -472,17 +500,44 @@ OpTree* do_pow(OpTree *a, OpTree *b)
         delete b;
         return do_oneover(a);
     }
-    if (a->op == 0 && b->op == 0) {
-        double val = pow(a->val, b->val);
-        delete a;
-        delete b;
-        return new OpTree(val);
-    }
     else {
         return new OpTree(OP_POW, a, simplify_terms(b));
     }
 }
 
+OpTree* do_voigt(OpTree *a, OpTree *b)
+{
+    if (a->op == 0 && b->op == 0) {
+        double val = humlik(a->val, b->val) / sqrt(M_PI);
+        delete a;
+        return new OpTree(val);
+    }
+    else
+        return new OpTree(OP_VOIGT, simplify_terms(a), simplify_terms(b));
+}
+
+OpTree* do_dvoigt_dx(OpTree *a, OpTree *b)
+{
+    if (a->op == 0 && b->op == 0) {
+        double val = humdev_dkdx(a->val, b->val) / sqrt(M_PI);
+        delete a;
+        return new OpTree(val);
+    }
+    else
+        return new OpTree(OP_DVOIGT_DX, simplify_terms(a), simplify_terms(b));
+}
+
+OpTree* do_dvoigt_dy(OpTree *a, OpTree *b)
+{
+    if (a->op == 0 && b->op == 0) {
+        double val = humdev_dkdy(a->val, b->val) / sqrt(M_PI);
+        delete a;
+        return new OpTree(val);
+    }
+    else
+        return new OpTree(OP_DVOIGT_DY, simplify_terms(a), simplify_terms(b));
+}
+
 ////////////////////////////////////////////////////////////////////////////
 
 struct MultFactor
@@ -643,50 +698,57 @@ void get_terms(OpTree *a, double multiplier, vector<MultTerm> &v)
     if (a->op == OP_MUL || a->op == OP_DIV || a->op == OP_SQRT 
             || a->op == OP_POW)
         a = simplify_factors(a);
-    if (a->op == OP_ADD) {
+
+    if (a->op == OP_ADD) {  // p + q
         get_terms(a->c1, multiplier, v);
         get_terms(a->c2, multiplier, v);
+        a->c1 = a->c2 = 0;
+        delete a;
     }
-    else if (a->op == OP_SUB) {
+    else if (a->op == OP_SUB) {  // p - q
         get_terms(a->c1, multiplier, v);
         get_terms(a->c2, -multiplier, v);
+        a->c1 = a->c2 = 0;
+        delete a;
     }
-    else if (a->op == OP_NEG) {
+    else if (a->op == OP_NEG) {  // - p
         get_terms(a->c1, -multiplier, v);
+        a->c1 = a->c2 = 0;
+        delete a;
     }
-    else if (a->op == OP_MUL && a->c1->op == 0) {
+    else if (a->op == OP_MUL && a->c1->op == 0) { // const * p
         get_terms(a->c2, multiplier*(a->c1->val), v);
+        a->c2 = 0;
+        delete a;
     }
-    else if (a->op == OP_DIV && a->c1->op == 0 && !is_eq(a->c1->val,1.)) {
+         // const / p for const != 1 (to avoid loop)
+    else if (a->op == OP_DIV && a->c1->op == 0 && a->c1->val != 1.) {
         get_terms(do_oneover(a->c2), multiplier*(a->c1->val), v);
+        a->c2 = 0;
+        delete a;
     }
-    else {
-        bool found = false;
-        if (a->op == 0)
-
-        for (vector<MultTerm>::iterator i = v.begin(); i != v.end(); ++i) 
-            if (i->t && *i->t == *a) {
-                i->k += multiplier;
-                found = true;
-                break;
-            }
-            else if (a->op == 0 && i->t && i->t->op == 0) {
+    else { // a can't be splitted
+        for (vector<MultTerm>::iterator i = v.begin(); i != v.end(); ++i) {
+            if (a->op == 0 && i->t && i->t->op == 0) {// number (not the first)
                 i->k += multiplier * a->val;
-                found = true;
-                break;
+                delete a;
+                return;
             }
-        if (!found) {
-            if (a->op == 0)
-                v.push_back(MultTerm(new OpTree(1.), multiplier * a->val));
-            else {
-                v.push_back(MultTerm(a, multiplier));
-                return; //don't delete a
+            if (i->t && *i->t == *a) { //token already in v
+                i->k += multiplier;
+                delete a;
+                return;
             }
         }
+        // we are here -- no first token of its kind
+        if (a->op == 0) { //add number
+            v.push_back(MultTerm(new OpTree(1.), multiplier * a->val));
+            delete a;
+        }
+        else {           // add token
+            v.push_back(MultTerm(a, multiplier));
+        }
     }
-    //we are here -- MultTerm(a,...) not created
-    a->c1 = a->c2 = 0;
-    delete a;
 }
 
 OpTree* simplify_terms(OpTree *a)
@@ -694,6 +756,11 @@ OpTree* simplify_terms(OpTree *a)
     // not handled:
     //        (x+y) * (x-y) == x^2 - y^2 
     //        (x+/-y)^2 == x^2 +/- 2xy + y^2
+    if (a->op == OP_MUL || a->op == OP_DIV || a->op == OP_SQRT 
+            || a->op == OP_POW)
+        return simplify_factors(a);
+    else if (!(a->op == OP_ADD || a->op == OP_SUB || a->op == OP_NEG))
+        return a;
 #ifdef DEBUG_SIMPLIFY
     cout << "simplify_terms() [<] " << a->str() << endl;
 #endif
@@ -751,25 +818,35 @@ fp get_constant_value(string const &s)
         assert(*(s.end()-1) == '}');
         string expr(s.begin()+1, s.end()-1);
         Data const* data = 0;
-        string::size_type in_pos = s.find(" in ");
-        if (in_pos != string::npos) {
+        string::size_type in_pos = expr.find(" in ");
+        if (in_pos != string::npos && in_pos+4 < expr.size()) {
+            string in_expr(expr, in_pos+4);
             int n;
-            expr.resize(in_pos);
-            string in_expr(expr, in_pos);
-            if (!parse(in_expr.c_str(), 
-                       "in" >> lexeme_d['@' >> uint_p[assign_a(n)]]).full)
-                throw ExecuteError("Syntax error in: " + in_expr);
-            data = AL->get_data(n);
+            if (parse(in_expr.c_str(), 
+                      *ch_p(' ') >> '@' >> uint_p[assign_a(n)] >> *ch_p(' ')
+                     ).full) {
+                data = AL->get_data(n);
+                expr.resize(in_pos);
+            }
+            else
+                throw ExecuteError("Syntax error near: `" + in_expr + "'");
         }
+        else if (AL->get_ds_count() == 1)
+            data = AL->get_data(0);
         return get_transform_expression_value(expr, data);
     }
-    else
-        return strtod(s.c_str(), 0);
+    else {
+        fp val = strtod(s.c_str(), 0);
+        if (val != 0. && fabs(val) < epsilon)
+            warn("Warning: Numeric literal 0 < |" + s + "| < epsilon=" 
+                    + S(epsilon) + ".");
+        return val;
+    }
 }
 
 /// returns array of trees, 
 /// first n=vars.size() derivatives and the last tree for value
-vector<OpTree*> calculate_deriv(const_iter_t const &i,
+vector<OpTree*> calculate_deriv(const_tm_iter_t const &i,
                                 vector<string> const &vars)
 {
     int len = vars.size();
@@ -798,75 +875,98 @@ vector<OpTree*> calculate_deriv(const_iter_t const &i,
 
     else if (i->value.id() == FuncGrammar::exptokenID)
     {
-        vector<OpTree*> arg = calculate_deriv(i->children.begin(), vars);
-        OpTree* (* do_op)(OpTree *) = 0;
-        OpTree* der = 0;
-        OpTree* larg = arg.back()->copy();
-        if (s == "sqrt") {
-            der = do_divide(new OpTree(0.5), do_sqrt(larg));
-            do_op = do_sqrt;
+        if (i->children.size() == 1) {
+            vector<OpTree*> arg = calculate_deriv(i->children.begin(), vars);
+            OpTree* (* do_op)(OpTree *) = 0;
+            OpTree* der = 0;
+            OpTree* larg = arg.back()->copy();
+            if (s == "sqrt") {
+                der = do_divide(new OpTree(0.5), do_sqrt(larg));
+                do_op = do_sqrt;
+            }
+            else if (s == "exp") {
+                der = do_exp(larg);
+                do_op = do_exp;
+            }
+            else if (s == "erfc") {
+                // d/dz erfc(z) = -2/sqrt(pi) * exp(-z^2) 
+                der = do_multiply(do_exp(do_neg(do_sqr(larg))), 
+                                  new OpTree(-2/sqrt(M_PI)));
+                do_op = do_erfc;
+            }
+            else if (s == "erf") {
+                // d/dz erf(z) =  2/sqrt(pi) * exp(-z^2) 
+                der = do_multiply(do_exp(do_neg(do_sqr(larg))), 
+                                  new OpTree(2/sqrt(M_PI)));
+                do_op = do_erf;
+            }
+            else if (s == "log10") {
+                OpTree *ln_10 = do_ln(new OpTree(10.));
+                der = do_oneover(do_multiply(larg, ln_10));
+                do_op = do_log10;
+            }
+            else if (s == "ln") {
+                der = do_oneover(larg);
+                do_op = do_ln;
+            }
+            else if (s == "sin") {
+                der = do_cos(larg);
+                do_op = do_sin;
+            }
+            else if (s == "cos") {
+                der = do_neg(do_sin(larg));
+                do_op = do_cos;
+            }
+            else if (s == "tan") {
+                der = do_oneover(do_sqr(do_cos(larg)));
+                do_op = do_tan;
+            }
+            else if (s == "atan") {
+                der = do_oneover(do_add(new OpTree(1.), do_sqr(larg)));
+                do_op = do_atan;
+            }
+            else if (s == "asin") {
+                OpTree *root_arg = do_sub(new OpTree(1.), do_sqr(larg));
+                der = do_oneover(do_sqrt(root_arg));
+                do_op = do_asin;
+            }
+            else if (s == "acos") {
+                OpTree *root_arg = do_sub(new OpTree(1.), do_sqr(larg));
+                der = do_divide(new OpTree(-1.), do_sqrt(root_arg));
+                do_op = do_acos;
+            }
+            else if (s == "lgamma") {
+                der = do_digamma(larg);
+                do_op = do_lgamma;
+            }
+            else
+                assert(0);
+            for (int k = 0; k < len; ++k) 
+                results[k] = do_multiply(der->copy(), arg[k]);
+            delete der;
+            results[len] = (*do_op)(arg[len]);
         }
-        else if (s == "exp") {
-            der = do_exp(larg);
-            do_op = do_exp;
-        }
-        else if (s == "erfc") {
-            // d/dz erfc(z) = -2/sqrt(pi) * exp(-z^2) 
-            der = do_multiply(do_exp(do_neg(do_sqr(larg))), 
-                              new OpTree(-2/sqrt(M_PI)));
-            do_op = do_erfc;
-        }
-        else if (s == "erf") {
-            // d/dz erf(z) =  2/sqrt(pi) * exp(-z^2) 
-            der = do_multiply(do_exp(do_neg(do_sqr(larg))), 
-                              new OpTree(2/sqrt(M_PI)));
-            do_op = do_erf;
-        }
-        else if (s == "log10") {
-            OpTree *ln_10 = do_ln(new OpTree(10.));
-            der = do_oneover(do_multiply(larg, ln_10));
-            do_op = do_log10;
-        }
-        else if (s == "ln") {
-            der = do_oneover(larg);
-            do_op = do_ln;
-        }
-        else if (s == "sin") {
-            der = do_cos(larg);
-            do_op = do_sin;
-        }
-        else if (s == "cos") {
-            der = do_neg(do_sin(larg));
-            do_op = do_cos;
-        }
-        else if (s == "tan") {
-            der = do_oneover(do_sqr(do_cos(larg)));
-            do_op = do_tan;
-        }
-        else if (s == "atan") {
-            der = do_oneover(do_add(new OpTree(1.), do_sqr(larg)));
-            do_op = do_atan;
-        }
-        else if (s == "asin") {
-            OpTree *root_arg = do_sub(new OpTree(1.), do_sqr(larg));
-            der = do_oneover(do_sqrt(root_arg));
-            do_op = do_asin;
-        }
-        else if (s == "acos") {
-            OpTree *root_arg = do_sub(new OpTree(1.), do_sqr(larg));
-            der = do_divide(new OpTree(-1.), do_sqrt(root_arg));
-            do_op = do_acos;
-        }
-        else if (s == "lgamma") {
-            der = do_digamma(larg);
-            do_op = do_lgamma;
+        else if (i->children.size() == 2) {
+            vector<OpTree*> 
+                left = calculate_deriv(i->children.begin(), vars),
+                right = calculate_deriv(i->children.begin() + 1, vars);
+            OpTree *d1=0, *d2=0;
+            if (s == "voigt") {
+                d1 = do_dvoigt_dx(left[len]->copy(), right[len]->copy());
+                d2 = do_dvoigt_dy(left[len]->copy(), right[len]->copy());
+                for (int k = 0; k < len; ++k) {
+                    results[k] = do_add(do_multiply(d1->copy(), left[k]),
+                                        do_multiply(d2->copy(), right[k]));
+                }
+                results[len] = do_voigt(left[len], right[len]);
+            }
+            else
+                assert(0);
+            delete d1;
+            delete d2;
         }
         else
             assert(0);
-        for (int k = 0; k < len; ++k) 
-            results[k] = do_multiply(der->copy(), arg[k]);
-        delete der;
-        results[len] = (*do_op)(arg[len]);
     }
 
     else if (i->value.id() == FuncGrammar::signargID)
