@@ -1,5 +1,6 @@
 // This file is part of fityk program. Copyright (C) Marcin Wojdyr
-// $Id: bfunc.cpp,v 1.6 2006/09/08 12:08:30 wojdyr Exp $
+// Licence: GNU General Public License version 2
+// $Id: bfunc.cpp 264 2007-03-01 10:10:54Z wojdyr $
 /// Built-in function definitions
 
 #include "bfunc.h"
@@ -211,8 +212,8 @@ const char *FuncGaussian::formula
 
 void FuncGaussian::more_precomputations() 
 { 
-    if (fabs(vv[2]) < EPSILON) 
-        vv[2] = EPSILON; 
+    if (fabs(vv[2]) < epsilon) 
+        vv[2] = epsilon; 
 }
 
 FUNC_CALCULATE_VALUE_BEGIN(Gaussian)
@@ -252,10 +253,10 @@ const char *FuncSplitGaussian::formula
 
 void FuncSplitGaussian::more_precomputations() 
 { 
-    if (fabs(vv[2]) < EPSILON) 
-        vv[2] = EPSILON; 
-    if (fabs(vv[3]) < EPSILON) 
-        vv[3] = EPSILON; 
+    if (fabs(vv[2]) < epsilon) 
+        vv[2] = epsilon; 
+    if (fabs(vv[3]) < epsilon) 
+        vv[3] = epsilon; 
 }
 
 FUNC_CALCULATE_VALUE_BEGIN(SplitGaussian)
@@ -306,8 +307,8 @@ const char *FuncLorentzian::formula
 
 void FuncLorentzian::more_precomputations()
 { 
-    if (fabs(vv[2]) < EPSILON) 
-        vv[2] = EPSILON; 
+    if (fabs(vv[2]) < epsilon) 
+        vv[2] = epsilon; 
 }
 
 FUNC_CALCULATE_VALUE_BEGIN(Lorentzian)
@@ -347,8 +348,8 @@ const char *FuncPearson7::formula
 
 void FuncPearson7::more_precomputations()
 { 
-    if (fabs(vv[2]) < EPSILON) 
-        vv[2] = EPSILON; 
+    if (fabs(vv[2]) < epsilon) 
+        vv[2] = epsilon; 
     if (vv.size() != 5)
         vv.resize(5);
     // not checking for vv[3]>0.5 nor even >0
@@ -415,10 +416,10 @@ const char *FuncSplitPearson7::formula
 
 void FuncSplitPearson7::more_precomputations()
 { 
-    if (fabs(vv[2]) < EPSILON) 
-        vv[2] = EPSILON; 
-    if (fabs(vv[3]) < EPSILON) 
-        vv[3] = EPSILON; 
+    if (fabs(vv[2]) < epsilon) 
+        vv[2] = epsilon; 
+    if (fabs(vv[3]) < epsilon) 
+        vv[3] = epsilon; 
     if (vv.size() != 8)
         vv.resize(8);
     // not checking for vv[3]>0.5 nor even >0
@@ -492,8 +493,8 @@ const char *FuncPseudoVoigt::formula
 
 void FuncPseudoVoigt::more_precomputations() 
 { 
-    if (fabs(vv[2]) < EPSILON) 
-        vv[2] = EPSILON; 
+    if (fabs(vv[2]) < epsilon) 
+        vv[2] = epsilon; 
 }
 
 FUNC_CALCULATE_VALUE_BEGIN(PseudoVoigt)
@@ -547,8 +548,8 @@ void FuncVoigt::more_precomputations()
     vv[4] = 1. / k;
     vv[5] = dkdy / k;
 
-    if (fabs(vv[2]) < EPSILON) 
-        vv[2] = EPSILON; 
+    if (fabs(vv[2]) < epsilon) 
+        vv[2] = epsilon; 
 }
 
 FUNC_CALCULATE_VALUE_BEGIN(Voigt)
@@ -618,8 +619,8 @@ void FuncVoigtA::more_precomputations()
         vv.resize(6);
     vv[4] = 1. / humlik(0, fabs(vv[3]));
 
-    if (fabs(vv[2]) < EPSILON) 
-        vv[2] = EPSILON; 
+    if (fabs(vv[2]) < epsilon) 
+        vv[2] = epsilon; 
 }
 
 FUNC_CALCULATE_VALUE_BEGIN(VoigtA)
@@ -698,7 +699,8 @@ FUNC_CALCULATE_VALUE_BEGIN(EMG)
     fp d = vv[3];
     fp fact = a*c*sqrt(2*M_PI)/(2*d);
     fp ex = exp(bx/d + c*c/(2*d*d));
-    fp erf_arg = bx/(M_SQRT2*c) + c/(M_SQRT2*d);
+    //fp erf_arg = bx/(M_SQRT2*c) + c/(M_SQRT2*d);
+    fp erf_arg = (bx/c + c/d) / M_SQRT2;
     fp t = fact * ex * (d >= 0 ? erfc(erf_arg) : -erfc(-erf_arg));
     //fp t = fact * ex * (d >= 0 ? 1-erf(erf_arg) : -1-erf(erf_arg));
 FUNC_CALCULATE_VALUE_END(t)
