@@ -1,5 +1,6 @@
 // This file is part of fityk program. Copyright (C) Marcin Wojdyr
-// $Id: cmd2.cpp,v 1.13 2006/09/12 00:28:45 wojdyr Exp $
+// Licence: GNU General Public License version 2
+// $Id: cmd2.cpp 265 2007-03-02 18:41:10Z wojdyr $
 
 /// big grammars in Spirit take a lot of time and memory to compile
 /// so they must be splitted into separate compilation units
@@ -32,7 +33,7 @@ using namespace std;
 namespace cmdgram {
 
 bool with_plus, deep_cp;
-string t, t2, output_redir;
+string t, t2, t3, output_redir;
 string prepared_info;
 bool info_append;
 int tmp_int, tmp_int2, ds_pref;
@@ -83,14 +84,17 @@ void do_import_dataset(char const*, char const*)
         }
         else { // there is only one and empty slot -- load data there
             AL->get_data(-1)->load_file(t, t2, vn); 
+            AL->view.set_datasets(vector1(AL->get_ds(0)));
             AL->view.fit();
             tmp_int = 0;
         }
     }
     else { // slot number was specified -- load data there
         AL->get_data(tmp_int)->load_file(t, t2, vn); 
-        if (AL->get_ds_count() == 1)
+        if (AL->get_ds_count() == 1) {
+            AL->view.set_datasets(vector1(AL->get_ds(0)));
             AL->view.fit();
+        }
     }
     AL->activate_ds(tmp_int);
     outdated_plot=true;  
@@ -116,25 +120,40 @@ void do_load_data_sum(char const*, char const*)
         dd.push_back(AL->get_data(*i));
     if (tmp_int == new_dataset) 
         tmp_int = AL->append_ds();
-    AL->get_data(tmp_int)->load_data_sum(dd);
+    AL->get_data(tmp_int)->load_data_sum(dd, t);
     AL->activate_ds(tmp_int);
     outdated_plot=true;  
 }
 
 void do_plot(char const*, char const*)
 {
-    if (tmp_int != -1)
-        AL->activate_ds(tmp_int);
+    if (vds.size() == 1 && vds[0] >= 0)
+        AL->activate_ds(vds[0]);
+    bool need_ds = false;
+    for (vector<string>::const_iterator i = vr.begin(); i != vr.end(); ++i)
+        if (i->empty())
+            need_ds = true;
+    if (need_ds) {
+        vector<DataWithSum*> dsds = get_datasets_from_indata();
+        //move active ds to the front
+        DataWithSum* ads = AL->get_ds(AL->get_active_ds_position());
+        vector<DataWithSum*>::iterator pos = find(dsds.begin(),dsds.end(), ads);
+        if (pos != dsds.end() && pos != dsds.begin()) {
+            *pos = dsds[0];
+            dsds[0] = ads;
+        }
+        AL->view.set_datasets(dsds);
+    }
     AL->view.parse_and_set(vr);
     getUI()->drawPlot(1, true);
-    outdated_plot=false;
+    outdated_plot = false;
 }
 
 void do_output_info(char const*, char const*)
 {
     prepared_info = strip_string(prepared_info);
     if (output_redir.empty())
-        mesg(prepared_info);
+        rmsg(prepared_info);
     else {
         ofstream os(output_redir.c_str(), 
                     ios::out | (info_append ? ios::app : ios::trunc));
@@ -152,6 +171,9 @@ void do_print_info(char const* a, char const* b)
     vector<Function*> const &functions = AL->get_functions(); 
     if (s.empty())
         m = "info about what?";
+    else if (s == "version") {
+        m = VERSION;
+    }
     else if (s == "variables") {
         if (variables.empty())
             m = "No variables found.";
@@ -209,13 +231,21 @@ void do_print_info(char const* a, char const* b)
                 m += "\n@" + S(i) + ": " + AL->get_data(i)->get_title();
     }
     else if (s[0] == '@') {
-        m = AL->get_data(tmp_int)->getInfo();
+        Data const* data = AL->get_data(tmp_int);
+        if (s.find(".title") != string::npos)
+            m = data->get_title();
+        else if (s.find(".filename") != string::npos)
+            m = data->get_filename();
+        else
+            m = data->getInfo();
     }
     else if (s == "view") {
         m = AL->view.str();
     }
     else if (s == "set")
         m = getSettings()->print_usage();
+    else if (startswith(s, "fit-history"))
+        m = FitMethodsContainer::getInstance()->param_history_info();
     else if (startswith(s, "fit"))
         m = getFit()->getInfo(get_datasets_from_indata());
     else if (startswith(s, "errors"))
@@ -234,11 +264,12 @@ void do_print_info(char const* a, char const* b)
         vector<DataWithSum*> v = get_datasets_from_indata();
         for (vector<DataWithSum*>::const_iterator i=v.begin(); i!=v.end(); ++i){
             m += "# " + (*i)->get_data()->get_title() + "\n";
-            m += (*i)->get_sum()->get_formula(!with_plus) + "\n";
+            bool gnuplot = getSettings()->get_e("formula-export-style") == 1;
+            m += (*i)->get_sum()->get_formula(!with_plus, gnuplot) + "\n";
         }
     }
     else if (s == "commands")
-        m = getUI()->getCommands().get_info();
+        m = getUI()->getCommands().get_info(with_plus);
     else if (startswith(s, "guess")) {
         vector<DataWithSum*> v = get_datasets_from_indata();
         for (vector<DataWithSum*>::const_iterator i = v.begin(); 
@@ -248,14 +279,23 @@ void do_print_info(char const* a, char const* b)
     prepared_info += "\n" + m;
 }
 
-void do_print_sum_info(char const* a, char const* b)
+void do_print_func(char const*, char const*)
 {
-    string s = string(a,b);
-    string m;
-    Sum const* sum = AL->get_sum(ds_pref);
-    m = s + ": "; 
-    vector<int> const &idx = (s == "F" ? sum->get_ff_idx() : sum->get_zz_idx());
-    for (vector<int>::const_iterator i = idx.begin(); i != idx.end(); ++i){
+    vector<string> const &names = AL->get_sum(ds_pref)->get_names(t2[0]);
+    if (tmp_int < 0)
+        tmp_int += names.size();
+    if (is_index(tmp_int, names)) 
+        prepared_info += "\n" + AL->find_function(names[tmp_int])
+            ->get_info(AL->get_variables(), AL->get_parameters(), with_plus);
+    else
+        prepared_info += "\nNot found.";
+}
+
+void do_print_sum_info(char const*, char const*)
+{
+    string m = t2 + ": "; 
+    vector<int> const &idx = AL->get_sum(ds_pref)->get_indices(t2[0]);
+    for (vector<int>::const_iterator i = idx.begin(); i != idx.end(); ++i) {
         Function const* f = AL->get_function(*i);
         if (with_plus)
             m += "\n" + f->get_info(AL->get_variables(), AL->get_parameters());
@@ -287,7 +327,8 @@ void do_print_deriv(char const* a, char const* b)
     prepared_info += "\n" + get_derivatives_str(s);
 }
 
-void do_print_debug_info(char const*, char const*)  { 
+void do_print_debug_info(char const*, char const*)  
+{ 
     string m;
     if (t == "idx") {   // show varnames and var_idx from VariableUser
         for (int i = 0; i < size(AL->get_functions()); ++i) 
@@ -313,7 +354,7 @@ void do_print_debug_info(char const*, char const*)  {
         Function const* f = AL->find_function(t);
         m = f->get_bytecode();
     }
-    mesg(m);
+    rmsg(m);
 }
 
 
@@ -335,7 +376,10 @@ void do_print_data_expr(char const*, char const*)
                 fp k = get_transform_expression_value(t2, (*i)->get_data());
                 if (i != v.begin())
                     s += "\n";
-                s += "in @" + S(m[*i]) + ": " + S(k);
+                s += "in @" + S(m[*i]); 
+                if (with_plus)
+                    s += " " + (*i)->get_data()->get_title();
+                s += ": " + S(k);
             }
         }
     }
@@ -396,14 +440,6 @@ Cmd2Grammar::definition<ScannerT>::definition(Cmd2Grammar const& /*self*/)
             )
         ;
 
-    ds_multiprefix
-        = eps_p [clear_a(vds)]
-        >> (lexeme_d['@' >> uint_p [push_back_a(vds)]
-            >> '.']
-           | str_p("@*.") [push_back_a(vds, all_datasets)]
-           )
-        ;
-
     ds_prefix
         = lexeme_d['@' >> uint_p [assign_a(ds_pref)] 
            >> '.']
@@ -413,7 +449,7 @@ Cmd2Grammar::definition<ScannerT>::definition(Cmd2Grammar const& /*self*/)
     compact_str
         = lexeme_d['\'' >> (+~ch_p('\''))[assign_a(t)] 
                    >> '\'']
-        | lexeme_d[+chset<>(anychar_p - chset<>(" \t\n\r;,"))] [assign_a(t)]
+        | lexeme_d[+chset<>(anychar_p - chset<>(" \t\n\r;,#"))] [assign_a(t)]
         ;
 
     type_name
@@ -435,15 +471,15 @@ Cmd2Grammar::definition<ScannerT>::definition(Cmd2Grammar const& /*self*/)
         ;
 
     dataset_handling
-        = dataset_nr >> ch_p('<') [clear_a(vn)] 
-          >> ((lexeme_d['@' >> uint_p [push_back_a(vn)]  //sum/duplicate
-                       ] % '+') [&do_load_data_sum]
-             | (compact_str                              //load from file
-                 >> lexeme_d[!(alpha_p >> *alnum_p)][assign_a(t2)]
-                 >> !(uint_p [push_back_a(vn)]
-                        % ',')
-               ) [&do_import_dataset]
-             )
+        = (dataset_nr >> ch_p('<') >> compact_str //load from file
+           >> lexeme_d[!(alpha_p >> *alnum_p)] [assign_a(t2)] [clear_a(vn)]
+           >> !(uint_p [push_back_a(vn)] 
+                % ',')
+          ) [&do_import_dataset]
+        | dataset_nr >> ch_p('=') [clear_a(vn)] [assign_a(t, empty)] //sum/dup
+          >> !(lexeme_d[lower_p >> +(alnum_p | '-' | '_')] [assign_a(t)])
+          >> (lexeme_d['@' >> uint_p [push_back_a(vn)]  
+                                           ] % '+') [&do_load_data_sum]
         | (existing_dataset_nr [clear_a(vt)]
            >> !('(' >> ((DataExpressionG
                         | ("*F(" >> DataExpressionG >> ")")
@@ -464,32 +500,41 @@ Cmd2Grammar::definition<ScannerT>::definition(Cmd2Grammar const& /*self*/)
         ;
     
     info_arg
-        = str_p("variables") [&do_print_info]
-        | VariableLhsG [&do_print_info]
-        | str_p("types") [&do_print_info]
-        | str_p("functions") [&do_print_info]
-        | str_p("datasets") [&do_print_info]
-        | (str_p("commands") >> optional_plus >> IntRangeG) [&do_list_commands] 
-        | str_p("commands") [&do_print_info] 
-        | str_p("view") [&do_print_info]
-        | str_p("set") [&do_print_info] 
-        | (str_p("fit") >> in_data) [&do_print_info] 
-        | (str_p("errors") >> in_data) [&do_print_info] 
-        | (str_p("peaks") >> in_data) [&do_print_info]
-        | (str_p("formula") >> in_data) [&do_print_info]
+        = (str_p("commands") >> IntRangeG) [&do_list_commands] 
+        |  ( str_p("version") 
+           | str_p("variables") 
+           | VariableLhsG 
+           | str_p("types") 
+           | str_p("functions") 
+           | str_p("datasets") 
+           | str_p("commands") 
+           | str_p("view") 
+           | str_p("set")) [&do_print_info] 
+        | ((str_p("fit-history") 
+            | "fit"
+            | "errors"
+            | "peaks"
+            | "formula"
+                    ) >> in_data) [&do_print_info] 
         | (str_p("guess") [clear_a(vr)]
            >> plot_range >> in_data) [&do_print_info]
         | type_name[&do_print_func_type]
         | (no_actions_d[DataExpressionG][assign_a(t2)] 
              >> in_data) [&do_print_data_expr]
         | FunctionLhsG [&do_print_info]
-        | ds_prefix >> (ch_p('F')|'Z') [&do_print_sum_info]
+        | ds_prefix >> (str_p("F")|"Z")[assign_a(t2)] 
+          >> ( ('[' >> int_p [assign_a(tmp_int)] >> ']') [&do_print_func] 
+             | eps_p [&do_print_sum_info]
+             )
         | (ds_prefix >> str_p("dF") >> '(' 
            >> no_actions_d[DataExpressionG][assign_a(t2)] 
            >> ')') [&do_print_sum_derivatives_info]
-        | existing_dataset_nr [&do_print_info]
-        | "debug" >> compact_str [&do_print_debug_info] //no output_redir
-        | "der " >> (+chset<>(anychar_p - chset<>(" \t\n\r;,#"))) [&do_print_deriv]
+        | (existing_dataset_nr 
+                >> (str_p(".filename") | ".title" | eps_p)) [&do_print_info]
+        | "debug " >> compact_str [&do_print_debug_info] //no output_redir
+        // this will eat also ',', because voigt(x,y) needs it
+        // unfortunatelly "i der x^2, sin(x)" is made impossible
+        | "der " >> (+chset<>(anychar_p - chset<>(";#"))) [&do_print_deriv]
         | eps_p [&do_print_info] 
         ;
 
@@ -521,9 +566,8 @@ Cmd2Grammar::definition<ScannerT>::definition(Cmd2Grammar const& /*self*/)
                 >> compact_str [assign_a(output_redir, t)]
                )
           ) [&do_output_info]
-        | (optional_suffix_p("p","lot") 
-                              [clear_a(vr)] [assign_a(tmp_int, minus_one)]
-          >> !existing_dataset_nr >> plot_range >> plot_range) [&do_plot]
+        | (optional_suffix_p("p","lot") [clear_a(vr)] 
+           >> plot_range >> plot_range >> in_data) [&do_plot]
         | guess [&do_guess]
         | (ds_prefix >> "title" >> '=' >> compact_str)[&set_data_title]
         | dataset_handling
