@@ -1,5 +1,6 @@
 // This file is part of fityk program. Copyright (C) Marcin Wojdyr
-// $Id: data.cpp,v 1.34 2006/09/12 00:28:45 wojdyr Exp $
+// Licence: GNU General Public License version 2
+// $Id: data.cpp 264 2007-03-01 10:10:54Z wojdyr $
 
 #include "common.h"
 #include "data.h" 
@@ -7,6 +8,7 @@
 #include "ui.h"
 #include "numfuncs.h"
 #include "datatrans.h" 
+#include "settings.h" 
 
 #include <math.h>
 #include <string.h>
@@ -91,19 +93,30 @@ void Data::clear()
     given_cols.clear();
     p.clear();
     active_p.clear();
+    has_sigma = false;
 }
 
 void Data::post_load()
 {
     if (p.empty())
         return;
-    if (!p[0].sigma) {
-        for (vector<Point>::iterator i = p.begin(); i < p.end(); i++) 
-            i->sigma = i->y > 1. ? sqrt (i->y) : 1.;
-        info(S(p.size()) + " points. No explicit std. dev. Set as sqrt(y)");
+    string inf = S(p.size()) + " points.";
+    if (!has_sigma) {
+        int dds = getSettings()->get_e("data-default-sigma");
+        if (dds == 's') {
+            for (vector<Point>::iterator i = p.begin(); i < p.end(); i++) 
+                i->sigma = i->y > 1. ? sqrt (i->y) : 1.;
+            inf += " No explicit std. dev. Set as sqrt(y)";
+        }
+        else if (dds == '1') {
+            for (vector<Point>::iterator i = p.begin(); i < p.end(); i++) 
+                i->sigma = 1.; 
+            inf += " No explicit std. dev. Set as equal 1.";
+        }
+        else
+            assert(0);
     }
-    else
-        info(S(p.size()) + " points.");
+    msg(inf);
     if (title.empty())
         title = get_file_basename(filename);
     update_active_p();
@@ -137,29 +150,88 @@ int Data::load_arrays(const vector<fp> &x, const vector<fp> &y,
     if (sigma.empty()) 
         for (size_t i = 0; i < size; ++i)
             p.push_back (Point (x[i], y[i]));
-    else
+    else {
         for (size_t i = 0; i < size; ++i)
             p.push_back (Point (x[i], y[i], sigma[i]));
+        has_sigma = true;
+    }
     sort(p.begin(), p.end());
     x_step = find_step();
     post_load();
     return p.size();
 }
 
-void Data::load_data_sum(vector<Data const*> const& dd)
+namespace {
+
+void apply_operation(vector<Point> &pp, string const& op)
+{
+    assert (!pp.empty());
+    assert (!op.empty());
+    if (op == "sum_same_x" || op == "avg_same_x") {
+        bool avg = (op == "avg_same_x");
+        int count_same = 1;
+        fp x0;
+        for (int i = pp.size() - 2; i >= 0; --i) {
+            if (count_same == 1)
+                x0 = pp[i+1].x;
+            if (is_eq(pp[i].x, x0)) {
+                pp[i].x += pp[i+1].x;
+                pp[i].y += pp[i+1].y;
+                pp[i].sigma += pp[i+1].sigma;
+                pp[i].is_active = pp[i].is_active || pp[i+1].is_active;
+                pp.erase(pp.begin() + i+1);
+                count_same++;
+                if (i > 0)
+                    continue;
+                else
+                    i = -1; // to change pp[0]
+            }
+            if (count_same > 1) {
+                pp[i+1].x /= count_same;
+                if (avg) {
+                    pp[i+1].y /= count_same;
+                    pp[i+1].sigma /= count_same;
+                }
+                count_same = 1;
+            }
+        }
+    }
+    else if (op == "fft") {
+        throw ExecuteError("Fourier Transform not implemented yet");
+    }
+    else if (op == "ifft") {
+        throw ExecuteError("Inverse FFT not implemented yet");
+    }
+    else
+        throw ExecuteError("Unknown dataset operation: " + op);
+}
+
+} // anonymous namespace
+
+void Data::load_data_sum(vector<Data const*> const& dd, string const& op)
 {
     assert(!dd.empty());
-    clear();
-    title = dd[0]->get_title();
+    // dd can contain this, we can't change p or title in-place.
+    std::vector<Point> new_p;
+    string new_title = dd[0]->get_title();
+    string new_filename = dd.size() == 1 ? dd[0]->get_filename() : "";
     for (vector<Data const*>::const_iterator i = dd.begin()+1; 
                                                           i != dd.end(); ++i)
-        title += " + " + (*i)->get_title();
+        new_title += " + " + (*i)->get_title();
     for (vector<Data const*>::const_iterator i = dd.begin(); 
                                                           i != dd.end(); ++i) {
         vector<Point> const& points = (*i)->points();
-        p.insert(p.end(), points.begin(), points.end());
+        new_p.insert(new_p.end(), points.begin(), points.end());
     }
-    sort(p.begin(), p.end());
+    sort(new_p.begin(), new_p.end());
+    if (!new_p.empty() && !op.empty())
+        apply_operation(new_p, op);
+        // data should be sorted after apply_operation()
+    clear();
+    title = new_title;
+    filename = new_filename;
+    p = new_p;
+    has_sigma = true;
     x_step = find_step();
     post_load();
 }
@@ -213,23 +285,23 @@ void Data::load_xy_filetype (ifstream& f, vector<int> const& columns)
     *           38.911500      352.431
     * delimiters: white spaces and  , : ;
      */
-    if (columns.size() == 1 || columns.size() > 3)
+    if (columns.size() == 1 || columns.size() > 3) //0, 2, or 3 columns allowed
         throw ExecuteError("If columns are specified, two or three of them"
                            " must be given (eg. 1,2,3)");
     vector<int> const& cols = columns.empty() ? vector2<int>(1, 2) : columns;
     vector<fp> xy;
+    has_sigma = (columns.size() == 3);
     int maxc = *max_element (cols.begin(), cols.end());
     int minc = *min_element (cols.begin(), cols.end());
-    if (minc < 1) {
-        warn ("Invalid column number: " + S(minc) 
-                + ". (First column is 1, column number has to be positive)");
+    if (minc < 0) {
+        warn ("Invalid (negative) column number: " + S(minc));
         return;
     }
     int not_enough_cols = 0, non_data_lines = 0, non_data_blocks = 0;
     bool prev_empty = false;
     //TODO: optimize loop below
-    //most of time (?) is spent in getline() in get_one_line_with_numbers()
-    while (get_one_line_with_numbers(f, xy)) {
+    //most of time (?) is spent in getline() in read_line_and_get_all_numbers()
+    while (read_line_and_get_all_numbers(f, xy)) {
         if (xy.empty()) {
             non_data_lines++;
             if (!prev_empty) {
@@ -246,21 +318,21 @@ void Data::load_xy_filetype (ifstream& f, vector<int> const& columns)
             continue;
         }
 
-        fp x = xy[cols[0] - 1];
-        fp y = xy[cols[1] - 1];
+        fp x = cols[0] == 0 ? p.size() : xy[cols[0] - 1];
+        fp y = cols[1] == 0 ? p.size() : xy[cols[1] - 1];
         if (cols.size() == 2)
-            p.push_back (Point (x, y));
+            p.push_back (Point(x, y));
         else {// cols.size() == 3
-            fp sig = xy[cols[2] - 1];
-            if (sig <= 0) 
+            fp sig = cols[2] == 0 ? p.size() : xy[cols[2] - 1];
+            if (sig > 0) 
+                p.push_back (Point(x, y, sig));
+            else
                 warn ("Point " + S(p.size()) + " has sigma = " + S(sig) 
                         + ". Point canceled.");
-            else
-                p.push_back (Point (x, y, sig));
         }
     }
     if (non_data_lines > 0)
-        info (S(non_data_lines) +" (not empty and not `#...') non-data lines "
+        msg (S(non_data_lines) +" (not empty and not `#...') non-data lines "
                 "in " + S(non_data_blocks) + " blocks.");
     if (not_enough_cols > 0)
         warn ("Less than " + S(maxc) + " numbers in " + S(not_enough_cols) 
@@ -269,19 +341,26 @@ void Data::load_xy_filetype (ifstream& f, vector<int> const& columns)
     x_step = find_step();
 }
 
+string Data::read_one_line_as_title(ifstream& f, int column)
+{
+    if (!f)
+        return "";
+    string s;
+    getline(f, s);
+    --column;
+    if (column >= 0) {
+        vector<string> v = split_string(s, " \t");
+        if (column < size(v)) 
+            return v[column];
+    }
+    return strip_string(s);
+}
+
 /// read first line as file's title, and than run load_xy_filetype()
 void Data::load_header_xy_filetype(ifstream& f, vector<int> const& columns)
 {
-    string s;
-    getline(f, s);
-    if (columns.size() > 1) {
-        int col = columns[1] - 1;
-        vector<string> v = split_string(s, " \t");
-        if (size(v) > col) 
-            title = v[col];
-    }
-    else
-        title = strip_string(s);
+    int title_col = columns.size() > 1 ? columns[1] : 0;
+    title = Data::read_one_line_as_title(f, title_col);
     load_xy_filetype(f, columns);
 }
 
@@ -374,7 +453,7 @@ void Data::load_rit_filetype (ifstream& f)
     */
 
     vector<fp> num;
-    bool r = get_one_line_with_numbers(f, num);
+    bool r = read_line_and_get_all_numbers(f, num);
     if (!r || num.size() < 2 ){
         warn ("Bad format in \"header\" of .rit file");
         return;
@@ -382,7 +461,7 @@ void Data::load_rit_filetype (ifstream& f)
     fp xmin = num[0];
     x_step = static_cast<int>(num[1] * 1e4) / 1e4; //only 4 digits after '.'
     vector<fp> ys;
-    while (get_one_line_with_numbers(f, ys)) {
+    while (read_line_and_get_all_numbers(f, ys)) {
         if (ys.size() == 0)
             warn ("Error when trying to read " + S (p.size() + 1) + ". point. " 
                     "Ignoring line.");
@@ -408,7 +487,6 @@ fp Data::get_y_at (fp x) const
 void Data::transform(const string &s) 
 {
     p = transform_data(s, p);
-    //TODO history
     sort(p.begin(), p.end());
     update_active_p();
 }
@@ -501,7 +579,7 @@ fp Data::find_step()
         return 0.;
 }
         
-int Data::get_one_line_with_numbers(istream &is, vector<fp>& result_numbers) 
+int Data::read_line_and_get_all_numbers(istream &is, vector<fp>& result_numbers)
 {
     // returns number of numbers in line
     result_numbers.clear();
