@@ -1,5 +1,6 @@
 // This file is part of fityk program. Copyright (C) Marcin Wojdyr
-// $Id: datatrans.cpp,v 1.38 2006/09/12 00:28:45 wojdyr Exp $
+// Licence: GNU General Public License version 2
+// $Id: datatrans.cpp 264 2007-03-01 10:10:54Z wojdyr $
 
 // the idea of VM is based on one of boost::spirit samples - vmachine_calc
 
@@ -71,7 +72,7 @@
 //
 // All computations are performed using real numbers, but using round for
 // comparisions should not be neccessary. Two numbers that differ less
-// than EPSILON (see: common.h) ie. abs(a-b)<EPSILON, are considered equal. 
+// than epsilon (see: common.h) ie. abs(a-b)<epsilon, are considered equal. 
 // Indices are also computed in real number domain, 
 // and if they are not integers, interpolation of two values
 // is taken (i.e of values with indices floor(idx) and ceil(idx) 
@@ -127,6 +128,7 @@
 //#include "datatrans.h"
 #include "datatrans2.h"
 #include "sum.h"
+#include "voigt.h"
 //#include "common.h"
 //#include "data.h"
 //#include "var.h"
@@ -139,7 +141,7 @@
 using namespace datatrans;
 
 //#include "ui.h"
-//#define DT_DEBUG(x) mesg(x);
+//#define DT_DEBUG(x) msg(x);
 
 #ifdef STANDALONE_DATATRANS
 
@@ -271,6 +273,7 @@ string dt_op(int op)
     OP_(NEG)   OP_(EXP)   OP_(SIN)   OP_(COS)  OP_(ATAN)  OP_(ABS)  OP_(ROUND) 
     OP_(TAN) OP_(ASIN) OP_(ACOS)
     OP_(LOG10) OP_(LN)  OP_(SQRT)  OP_(POW)  
+    OP_(GAMMA) OP_(LGAMMA) OP_(VOIGT)
     OP_(ADD)   OP_(SUB)   OP_(MUL)   OP_(DIV)  OP_(MOD)
     OP_(MIN2)   OP_(MAX2) OP_(RANDNORM) OP_(RANDU)    
     OP_(VAR_X) OP_(VAR_Y) OP_(VAR_S) OP_(VAR_A) 
@@ -284,7 +287,7 @@ string dt_op(int op)
     OP_(DO_ONCE) OP_(RESIZE) OP_(ORDER) OP_(DELETE) OP_(BEGIN) OP_(END) 
     OP_(END_AGGREGATE) OP_(AGCONDITION) 
     OP_(AGSUM) OP_(AGMIN) OP_(AGMAX) OP_(AGAREA) OP_(AGAVG) OP_(AGSTDDEV)
-    OP_(PARAMETERIZED) OP_(PLIST_BEGIN) OP_(PLIST_END)
+    OP_(PARAMETERIZED) OP_(PLIST_BEGIN) OP_(PLIST_SEP) OP_(PLIST_END)
     OP_(FUNC) OP_(SUM_F) OP_(SUM_Z) OP_(NUMAREA) OP_(FINDX) OP_(FIND_EXTR)
     return S(op);
 };
@@ -362,7 +365,7 @@ vector<int>::const_iterator find_end(vector<int>::const_iterator i)
 template<typename T>
 fp get_var_with_idx(fp idx, vector<Point> const& points, T Point::*t)
 {
-    if (idx < 0 || idx > points.size()-1)
+    if (idx < 0 || idx+1 > points.size())
         return 0.;
     else if (is_eq(idx, iround(idx)))
         return points[iround(idx)].*t;
@@ -570,6 +573,11 @@ bool execute_code(int n, int &M, vector<fp>& stack,
                 STACK_OFFSET_CHANGE(-1)
                 STACK_OP *stackPtr = pow(*stackPtr, *(stackPtr+1));
                 break;
+            case OP_VOIGT:
+                STACK_OFFSET_CHANGE(-1)
+                STACK_OP *stackPtr = humlik(*stackPtr, *(stackPtr+1)) 
+                                                               / sqrt(M_PI);
+                break;
 
             // comparisions
             case OP_LT:
@@ -815,6 +823,7 @@ bool execute_code(int n, int &M, vector<fp>& stack,
     return return_value;
 }
 
+#if 0
 // not tested !!!!!!
 // this function could be used for optimization of data transformations
 // but optimization is not neccessary (?)
@@ -1196,6 +1205,7 @@ void multipoint_execute_code(int M, vector<fp>& stack,
         }
     }
 }
+#endif
 
 
 /// change  AGSUM X ... X END_AGGREGATE  to  NUMBER INDEX 
@@ -1285,6 +1295,9 @@ void execute_vm_code(const vector<Point> &old_points, vector<Point> &new_points)
 {
     vector<fp> stack(stack_size);
     int M = (int) new_points.size();
+    for (vector<ParameterizedFunction*>::iterator i = parameterized.begin();
+            i != parameterized.end(); ++i)
+        (*i)->prepare_parameters(old_points);
     replace_aggregates(M, old_points, code, code.begin());
     // first execute one-time operations: sorting, x[15]=3, etc. 
     // n==M => one-time op.
@@ -1353,26 +1366,33 @@ bool is_data_dependent_expression(string const& s)
     return is_data_dependent_code(code);
 }
 
-
 fp get_transform_expression_value(string const &s, Data const* data)
 {
-    static vector<fp> stack(stack_size);
-    vector<Point> const no_points;
-    vector<Point> const& points = data ? data->points() : no_points;
     clear_parse_vecs();
     // First compile string... puts result into code etc.
     parse_info<> result = parse(s.c_str(), DataExpressionG, space_p);
     if (!result.full) 
         throw ExecuteError("Syntax error in expression: " + s);
     if (!data && is_data_dependent_code(code))
-        throw ExecuteError("Expression depends on data points: " + s);
+        throw ExecuteError("Dataset is not specified and the expression "
+                           "depends on it: " + s);
+    vector<Point> const no_points;
+    return get_transform_expr_value(code, data ? data->points() : no_points);
+}
+
+fp get_transform_expr_value(vector<int>& code_, vector<Point> const& points)
+{
+    static vector<fp> stack(stack_size);
     int M = (int) points.size();
     vector<Point> new_points = points;
-    replace_aggregates(M, points, code, code.begin());
+    for (vector<int>::const_iterator i = code_.begin(); i != code_.end(); ++i) 
+        if (*i == OP_PARAMETERIZED) 
+            parameterized[*(i+1)]->prepare_parameters(points);
+    replace_aggregates(M, points, code_, code_.begin());
     // n==M => one-time op.
-    bool t = execute_code(M, M, stack, points, new_points, code);
+    bool t = execute_code(M, M, stack, points, new_points, code_);
     if (t)  
-        throw ExecuteError("Expression depends on undefined `n' index: " + s);
+        throw ExecuteError("Expression depends on undefined `n' index.");
     return stack.front();
 }
 
@@ -1394,13 +1414,13 @@ bool get_dt_code(string const& s, vector<int>& code_, vector<fp>& numbers_)
 }
 
 fp get_value_for_point(vector<int> const& code_, vector<fp> const& numbers_,
-                       Point const& p)
+                       fp x, fp y)
 {
     static vector<fp> stack(stack_size);
     static vector<Point> points(1); 
     static vector<Point> new_points(1); 
     numbers = numbers_;
-    points[0] = new_points[0] = p;
+    points[0] = new_points[0] = Point(x, y);
     int M = 1;
     execute_code(0, M, stack, points, new_points, code_);
     return stack.front();
@@ -1419,6 +1439,9 @@ vector<fp> get_all_point_expressions(string const &s, Data const* data,
     int M = (int) points.size();
     vector<Point> new_points = points;
     vector<fp> stack(stack_size);
+    for (vector<ParameterizedFunction*>::iterator i = parameterized.begin();
+            i != parameterized.end(); ++i)
+        (*i)->prepare_parameters(points);
     replace_aggregates(M, points, code, code.begin());
     for (int i = 0; i < M; ++i) {
         if (only_active && !points[i].is_active)
