@@ -1,5 +1,6 @@
 // This file is part of fityk program. Copyright (C) 2003 Stefan Krumm  
-// $Id: fileroutines.cpp,v 1.4 2005/05/13 18:54:48 wojdyr Exp $
+// Licence: GNU General Public License version 2
+// $Id: fileroutines.cpp 264 2007-03-01 10:10:54Z wojdyr $
 
 
 #include <fstream>
@@ -11,7 +12,7 @@
 
 void load_siemensbruker_filetype (std::string filename, Data *data)
 {
-	info("entered SiemensBruker loading: "+ filename);
+	vmsg("entered SiemensBruker loading: "+ filename);
 
 	float biggest=0, smallest=99999, average=0;
   	char ctest[140];
