@@ -1,5 +1,6 @@
 // This file is part of fityk program. Copyright (C) Marcin Wojdyr
-// $Id: func.cpp,v 1.35 2006/09/07 16:07:19 wojdyr Exp $
+// Licence: GNU General Public License version 2
+// $Id: func.cpp 264 2007-03-01 10:10:54Z wojdyr $
 
 #include "common.h"
 #include "func.h"
@@ -164,7 +165,7 @@ vector<string> Function::get_all_types()
 string Function::get_formula(int n)
 {
     assert (n >= 0);
-    int nb = sizeof(builtin_formulas)/sizeof(builtin_formulas[0]);
+    int nb = sizeof(builtin_formulas) / sizeof(builtin_formulas[0]);
     if (n < nb)
         return builtin_formulas[n];
     UdfContainer::UDF const* udf = UdfContainer::get_udf(n - nb);
@@ -189,7 +190,7 @@ string Function::get_formula(string const& type)
 /// returns: 2 if built-in, in C++, 1 if buit-in UDF, 0 if defined by user
 int Function::is_builtin(int n)
 {
-    int nb = sizeof(builtin_formulas)/sizeof(builtin_formulas[0]);
+    int nb = sizeof(builtin_formulas) / sizeof(builtin_formulas[0]);
     assert (n >= 0 && n < nb + size(UdfContainer::udfs));
     if (n < nb)
         return 2;
@@ -379,8 +380,7 @@ fp Function::numarea(fp x1, fp x2, int nsteps) const
 /// search x in [x1, x2] for which %f(x)==val, 
 /// x1, x2, val: f(x1) <= val <= f(x2) or f(x2) <= val <= f(x1)
 /// bisection + Newton-Raphson
-fp Function::find_x_with_value(fp x1, fp x2, fp val, 
-                               fp xacc, int max_iter) const
+fp Function::find_x_with_value(fp x1, fp x2, fp val, int max_iter) const
 {
     fp y1 = calculate_value(x1) - val;
     fp y2 = calculate_value(x2) - val;
@@ -401,7 +401,7 @@ fp Function::find_x_with_value(fp x1, fp x2, fp val,
     fp t = (x1 + x2) / 2.;
     for (int i = 0; i < max_iter; ++i) {
         //check if converged
-        if (fabs(x1-x2) < xacc)
+        if (is_eq(x1, x2))
             return (x1+x2) / 2.;
 
         // calculate f and df
@@ -436,7 +436,7 @@ fp Function::find_x_with_value(fp x1, fp x2, fp val,
 }
 
 /// finds root of derivative, using bisection method
-fp Function::find_extremum(fp x1, fp x2, fp xacc, int max_iter) const
+fp Function::find_extremum(fp x1, fp x2, int max_iter) const
 {
     int n = 0;
     for (vector<Multi>::const_iterator j = multi.begin(); j != multi.end(); ++j)
@@ -482,7 +482,7 @@ fp Function::find_extremum(fp x1, fp x2, fp xacc, int max_iter) const
             x2 = t;
 
         //check if converged
-        if (fabs(x1-x2) < xacc)
+        if (is_eq(x1, x2))
             return (x1+x2) / 2.;
     }
     throw ExecuteError("The search has not converged in " + S(max_iter) 
@@ -501,7 +501,8 @@ void initialize_udfs()
 "GaussianA(area, center, hwhm) = Gaussian(area/hwhm/sqrt(pi/ln(2)), center, hwhm)\n"
 "LorentzianA(area, center, hwhm) = Lorentzian(area/hwhm/pi, center, hwhm)\n"
 "Pearson7A(area, center, hwhm, shape=2) = Pearson7(area/(hwhm*exp(lgamma(shape-0.5)-lgamma(shape))*sqrt(pi/(2^(1/shape)-1))), center, hwhm, shape)\n"
-"PseudoVoigtA(area, center, hwhm, shape=0.5) = GaussianA(area*(1-shape), center, hwhm) + LorentzianA(area*shape, center, hwhm)",
+"PseudoVoigtA(area, center, hwhm, shape=0.5) = GaussianA(area*(1-shape), center, hwhm) + LorentzianA(area*shape, center, hwhm)\n"
+"ExpDecay(a=0, t=1) = a*exp(-x/t)",
   "\n");
     udfs.clear();
     for (vector<string>::const_iterator i = formulae.begin(); 
@@ -534,11 +535,11 @@ vector<OpTree*> make_op_trees(string const& formula)
     return op_trees;
 }
 
-void check_fudf_rhs(string const& formula, vector<string> const& lhs_vars)
+void check_fudf_rhs(string const& rhs, vector<string> const& lhs_vars)
 {
-    if (formula.empty())
+    if (rhs.empty())
         throw ExecuteError("No formula");
-    tree_parse_info<> info = ast_parse(formula.c_str(), FuncG, space_p);
+    tree_parse_info<> info = ast_parse(rhs.c_str(), FuncG, space_p);
     if (!info.full)
         throw ExecuteError("Syntax error in formula");
     vector<string> vars = find_tokens_in_ptree(FuncGrammar::variableID, info);
@@ -553,6 +554,25 @@ void check_fudf_rhs(string const& formula, vector<string> const& lhs_vars)
         }
 }
 
+void check_rhs(string const& rhs, vector<string> const& lhs_vars)
+{
+    string::size_type t = rhs.find_first_not_of(" \t\r\n");
+    if (t != string::npos && isupper(rhs[t])) { //compound
+        parse_info<> info = parse(rhs.c_str(), 
+                               ((lexeme_d[(upper_p >> +alnum_p)] >> '('  
+                                 >> (no_actions_d[FuncG]  % ',') >> ')') % '+'),
+                               space_p);
+        if (!info.full)
+            throw ExecuteError("Syntax error in compound formula.");
+        vector<string> rf = get_cpd_rhs_components(rhs, false);
+        for (vector<string>::const_iterator i = rf.begin(); i != rf.end(); ++i) 
+            check_cpd_rhs_function(*i, lhs_vars);
+    } 
+    else { //udf, not compound
+        check_fudf_rhs(rhs, lhs_vars);
+    }
+}
+
 void define(std::string const &formula)
 {
     string type = Function::get_typename_from_formula(formula);
@@ -565,14 +585,7 @@ void define(std::string const &formula)
         else if (!islower((*i)[0]))
             throw ExecuteError("Improper variable: " + *i);
     }
-    if (is_compounded(formula)) {
-        vector<string> rf = get_cpd_rhs_components(formula);
-        for (vector<string>::const_iterator i = rf.begin(); i != rf.end(); ++i) 
-            check_cpd_rhs_function(*i, lhs_vars);
-    } 
-    else {
-        check_fudf_rhs(Function::get_rhs_from_formula(formula), lhs_vars);
-    }
+    check_rhs(Function::get_rhs_from_formula(formula), lhs_vars);
     if (is_defined(type) && !get_udf(type)->is_builtin) { 
         //defined, but can be undefined; don't undefine function implicitely
         throw ExecuteError("Function `" + type + "' is already defined. "
@@ -596,7 +609,7 @@ void undefine(std::string const &type)
                                                   j != udfs.end(); ++j) {
                 if (!j->is_builtin)
                     continue;
-                vector<string> rf = get_cpd_rhs_components(j->formula);
+                vector<string> rf = get_cpd_rhs_components(j->formula, true);
                 for (vector<string>::const_iterator k = rf.begin();
                                                           k != rf.end(); ++k) {
                     if (Function::get_typename_from_formula(*k) == type)
@@ -648,15 +661,15 @@ void check_cpd_rhs_function(std::string const& fun,
             if ((*k)[0] != '~' && (*k)[0] != '{' && (*k)[0] != '$' 
                     && (*k)[0] != '%' && !contains_element(lhs_vars, *k))
                 throw ExecuteError("Improper variable given in parameter " 
-                                + S(k-vars.begin()+1) + " of "+ t + ": " + *k);
+                              + S(j-gvars.begin()+1) + " of "+ t + ": " + *k);
     }
 }
 
 /// find components of RHS (split sum "A() + B() + ...")
-vector<string> get_cpd_rhs_components(string const &formula)
+vector<string> get_cpd_rhs_components(string const &formula, bool full)
 {
     vector<string> result;
-    string::size_type pos = formula.rfind('=') + 1, 
+    string::size_type pos = (full ? formula.rfind('=') + 1 : 0), 
                       rpos = 0;
     while (pos != string::npos) {
         rpos = find_matching_bracket(formula, formula.find('(', pos));
@@ -681,7 +694,7 @@ CompoundFunction::CompoundFunction(string const &name, string const &type,
     for (int j = 0; j != nv; ++j) 
         vmgr.assign_variable(varnames[j], ""); // mirror variables
 
-    vector<string> rf = UdfContainer::get_cpd_rhs_components(type_formula);
+    vector<string> rf = UdfContainer::get_cpd_rhs_components(type_formula,true);
     for (vector<string>::iterator i = rf.begin(); i != rf.end(); ++i) {
         for (int j = 0; j != nv; ++j) {
             replace_words(*i, type_var_names[j], vmgr.get_variable(j)->xname);
