@@ -1,5 +1,6 @@
 // This file is part of fityk program. Copyright (C) Marcin Wojdyr
-// $Id: mgr.cpp,v 1.2 2006/09/07 19:16:11 wojdyr Exp $
+// Licence: GNU General Public License version 2
+// $Id: mgr.cpp 264 2007-03-01 10:10:54Z wojdyr $
 
 #include "mgr.h"
 #include "common.h"
@@ -10,6 +11,7 @@
 #include "func.h"
 #include "sum.h"
 #include "settings.h"
+#include "logic.h" //VariableManager::get_or_make_variable() handles @0.F[1].a
 
 #include <stdlib.h>
 #include <ctype.h>
@@ -62,12 +64,14 @@ string VariableManager::get_or_make_variable(string const& func)
     if (parse(func.c_str(), VariableLhsG).full) // $foo
         return string(func, 1);
     else if (parse(func.c_str(), 
-                   FunctionLhsG [assign_a(tmp1)] 
-                       >> '[' >> 
-                        lexeme_d[alpha_p >> *(alnum_p|'_')][assign_a(tmp2)]
-                       >> ']')
-                  .full) {                     // %bar[bleh]
-        return get_func_param(tmp1, tmp2);
+                   (FunctionLhsG 
+                   | !lexeme_d['@' >> uint_p >> '.'] 
+                     >> (str_p("F[")|"Z[") >> int_p >> ch_p(']') 
+                   ) [assign_a(tmp1)] 
+                   >> '.' >> 
+                   lexeme_d[alpha_p >> *(alnum_p|'_')][assign_a(tmp2)]
+                  ).full) {                     // %bar.bleh
+        return AL->find_function_any(tmp1)->get_param_varname(tmp2);
     }
     else                                       // anything else
         return assign_variable("", func);
@@ -153,8 +157,18 @@ string VariableManager::assign_variable(string const &name, string const &rhs)
             val_str.erase(pos);
         }
         fp val = get_constant_value(val_str);
-        parameters.push_back(val);
-        int nr = parameters.size() - 1;
+        int nr;
+
+        // avoid changing order of parameters in case of "$_1 = ~1.23"
+        int old_pos = find_variable_nr(name);
+        if (old_pos != -1 && variables[old_pos]->is_simple()) {
+            nr = variables[old_pos]->get_nr();
+            parameters[nr] = val; //variable at old_pos will be deleted soon
+        }
+        else {
+            nr = parameters.size();
+            parameters.push_back(val);
+        }
         var = new Variable(nonempty_name, nr);
         if (!domain_str.empty()) 
             parse_and_set_domain(var, domain_str);
@@ -165,7 +179,10 @@ string VariableManager::assign_variable(string const &name, string const &rhs)
             throw ExecuteError("variable can't depend on x.");
         for (vector<string>::const_iterator i = vars.begin(); 
                                                 i != vars.end(); i++) 
-            if ((*i)[0]!='~' && (*i)[0]!='{' && (*i)[0]!='$' && (*i)[0]!='%')
+            if ((*i)[0]!='~' && (*i)[0]!='{' && (*i)[0]!='$' && (*i)[0]!='%'
+                && (*i)[0]!='@' 
+                && (((*i)[0]!='F' && (*i)[0]!='Z')
+                                        || i->size() < 2 || (*i)[1]!='['))
                 throw ExecuteError("`" + *i + "' can't be used as variable.");
         vector<OpTree*> op_trees = calculate_deriv(root, vars);
         // ~14.3 -> $var4
@@ -219,7 +236,7 @@ void VariableManager::remove_unreferred()
 {
     // remove auto-delete marked variables, which are not referred by others
     for (int i = variables.size()-1; i >= 0; --i)
-        if (variables[i]->auto_delete) {
+        if (variables[i]->is_auto_delete()) {
             if (!is_variable_referred(i)) {
                 delete variables[i];
                 variables.erase(variables.begin() + i);
@@ -233,7 +250,7 @@ void VariableManager::remove_unreferred()
             i != functions.end(); ++i) {
         (*i)->set_var_idx(variables);
     }
-    // remove unreffered parameters
+    // remove unreferred parameters
     for (int i = size(parameters)-1; i >= 0; --i) {
         bool del=true;
         for (int j = 0; j < size(variables); ++j)
@@ -353,6 +370,31 @@ void VariableManager::delete_funcs(vector<string> const &names)
         (*i)->find_function_indices();
 }
 
+bool VariableManager::is_function_referred(int n) const
+{
+    for (vector<Sum*>::const_iterator i = sums.begin(); i != sums.end(); ++i) {
+        if (contains_element((*i)->get_ff_idx(), n)
+                || contains_element((*i)->get_zz_idx(), n))
+            return true;
+    }
+    return false;
+}
+
+void VariableManager::auto_remove_functions()
+{
+    int func_size = functions.size();
+    for (int i = func_size - 1; i >= 0; --i)
+        if (functions[i]->is_auto_delete() && !is_function_referred(i)) {
+            delete functions[i];
+            functions.erase(functions.begin() + i);
+        }
+    if (func_size != size(functions)) {
+        remove_unreferred();
+        for (vector<Sum*>::iterator i = sums.begin(); i != sums.end(); ++i)
+            (*i)->find_function_indices();
+    }
+}
+
 void VariableManager::delete_funcs_and_vars(vector<string> const &xnames)
 {
     vector<string> vars, funcs;
@@ -370,7 +412,8 @@ void VariableManager::delete_funcs_and_vars(vector<string> const &xnames)
 }
 
 
-int VariableManager::find_function_nr(string const &name) {
+int VariableManager::find_function_nr(string const &name) 
+{
     string only_name = !name.empty() && name[0]=='%' ? string(name,1) : name;
     for (int i = 0; i < size(functions); ++i)
         if (functions[i]->name == only_name)
@@ -378,7 +421,8 @@ int VariableManager::find_function_nr(string const &name) {
     return -1;
 }
 
-const Function* VariableManager::find_function(string const &name) {
+const Function* VariableManager::find_function(string const &name) 
+{
     int n = find_function_nr(name);
     if (n == -1)
         throw ExecuteError("undefined function: " 
@@ -386,14 +430,16 @@ const Function* VariableManager::find_function(string const &name) {
     return functions[n];
 }
 
-int VariableManager::find_variable_nr(string const &name) {
+int VariableManager::find_variable_nr(string const &name) 
+{
     for (int i = 0; i < size(variables); ++i)
         if (variables[i]->name == name)
             return i;
     return -1;
 }
 
-Variable const* VariableManager::find_variable(string const &name) {
+Variable const* VariableManager::find_variable(string const &name) 
+{
     int n = find_variable_nr(name);
     if (n == -1)
         throw ExecuteError("undefined variable: $" + name);
@@ -433,13 +479,10 @@ void VariableManager::use_external_parameters(vector<fp> const &ext_param)
         (*i)->do_precomputations(variables);
 }
 
-void VariableManager::put_new_parameters(vector<fp> const &aa, 
-                                         string const &/*method*/,
-                                         bool change)
+void VariableManager::put_new_parameters(vector<fp> const &aa)
 {
-    //TODO history
-    if (change)
-        parameters = aa;
+    for (size_t i = 0; i < min(aa.size(), parameters.size()); ++i)
+        parameters[i] = aa[i];
     use_parameters();
 }
 
@@ -502,9 +545,7 @@ vector<string> VariableManager::make_varnames(string const &function,
                                               vector<string> const &vars) 
 {
     vector<string> varnames;
-    if (vars.empty())
-        return varnames;
-    bool has_eq = (vars[0].find('=') != string::npos);
+    bool has_eq = (vars.empty() || vars[0].find('=') != string::npos);
     for (vector<string>::const_iterator i = vars.begin(); i != vars.end(); ++i) 
         if ((i->find('=') != string::npos) != has_eq)
             throw ExecuteError("Either use keywords for all parameters"
@@ -541,7 +582,7 @@ string VariableManager::do_assign_func(Function* func)
             delete functions[i];
             functions[i] = func;
             if (!silent)
-                mesg("New function %" + func->name + " replaced the old one.");
+                msg("New function %" + func->name + " replaced the old one.");
             remove_unreferred();
             found = true;
             break;
@@ -550,7 +591,7 @@ string VariableManager::do_assign_func(Function* func)
     if (!found) {
         functions.push_back(func);
         if (!silent)
-            info("New function %" + func->name + " was created.");
+            msg("New function %" + func->name + " was created.");
     }
     func->do_precomputations(variables);
     return func->name;
@@ -611,12 +652,6 @@ void VariableManager::substitute_func_param(string const &name,
     remove_unreferred();
 }
 
-string VariableManager::get_func_param(string const &name, string const &param)
-{
-    Function const* k = find_function(name);
-    return k->get_var_name(k->get_param_nr(param));
-}
-
 fp VariableManager::variation_of_a (int n, fp variat) const
 {
     assert (0 <= n && n < size(get_parameters()));
